unit MabHxBtn;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TMabHexButton = class(TCustomControl)
  private
    { Private declarations }
    procedure HexagonIn(ARect: TRect; var AHex: array of TPoint);
  protected
    { Protected declarations }
    OuterHex: array[0..5] of TPoint;
    InnerHex: array[0..5] of TPoint;
    RegionHex: array[0..5] of TPoint;
    Region: HRGN;

    FBitmap: TBitmap;
    FDown: Boolean;

    procedure WMSize(var Mesg: TWMSize); message WM_SIZE;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure Paint; override;

    procedure SetBitmap(ABitmap: TBitmap);
    procedure SetDown(ADown: Boolean);
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property Bitmap: TBitmap read FBitmap write SetBitmap;
    property Cursor;
    property Down: Boolean read FDown write SetDown;
    property Enabled;
    property ParentShowHint;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

implementation

constructor TMabHexButton.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);

   Region := 0;
   Height := 33;
   Width := 33;

   FBitmap := TBitmap.Create;
   FDown := False;
end;

destructor TMabHexButton.Destroy;
begin
   if Region <> 0 then
      DeleteObject(Region);

   FBitmap.Free;

   inherited Destroy;
end;

procedure TMabHexButton.SetDown(ADown: Boolean);
begin
   if ADown <> FDown then
   begin
      FDown := ADown;
      Invalidate;
   end;
end;

procedure TMabHexButton.SetBitmap(ABitmap: TBitmap);
begin
   with FBitmap do
   begin
      Assign(ABitmap);
      if Empty then Dormant;
   end;
   Invalidate;
end;

procedure TMabHexButton.HexagonIn(ARect: TRect; var AHex: array of TPoint);
var
   w, d: Integer;
begin
   w := ARect.Right - ARect.Left;
   d := (ARect.Bottom - ARect.Top) div 3;

   AHex[0] := Point(ARect.Left + w div 2, ARect.Top);
   AHex[1] := Point(ARect.Right, ARect.Top + d);
   AHex[2] := Point(ARect.Right, ARect.Bottom - d);
   AHex[3] := Point(ARect.Left + w div 2, ARect.Bottom);
   AHex[4] := Point(ARect.Left, ARect.Bottom - d);
   AHex[5] := Point(ARect.Left, ARect.Top + d);
end;

procedure TMabHexButton.WMSize(var Mesg: TWMSize);
begin
   inherited;

   if Region <> 0 then
      DeleteObject(Region);

   HexagonIn(Rect(3, 3, Width - 3, Height - 3), InnerHex);
   HexagonIn(Rect(2, 2, Width - 2, Height - 2), OuterHex);
   HexagonIn(Rect(0, 0, Width, Height), RegionHex);

   Region := CreatePolygonRgn(RegionHex, 6, WINDING);
   SetWindowRgn(Handle, Region, True);
end;

procedure TMabHexButton.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
   inherited MouseDown(Button, Shift, X, Y);
   if (Button = mbLeft) and Enabled and not FDown then
   begin
      FDown := True;
      Repaint;
   end;
end;

procedure TMabHexButton.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
  DoClick: Boolean;
begin
   inherited MouseUp(Button, Shift, X, Y);

   DoClick := (X >= 0) and (X < ClientWidth) and (Y >= 0) and (Y <= ClientHeight);
   Down := not Down;

   if DoClick then Click;
end;

procedure TMabHexButton.Paint;
var
   bx, by: Integer;
begin
   inherited Paint;

   if Canvas.ClipRect.Top <> Canvas.ClipRect.Bottom then
   with Canvas do
   begin
      Pen.Style := psClear;
      Brush.Style := bsSolid;

      Pen.Color := clWindowFrame;
      Brush.Color := clBtnFace;
      Polygon(OuterHex);

      if not FBitmap.Empty then
      begin
         bx := (Width - FBitmap.Width) div 2 + 1;
         by := (Height - FBitmap.Height) div 2;

         if Down then
         begin
            bx := bx + 1;
            by := by + 1;
         end;
         BrushCopy(
            Rect(bx, by, bx + FBitmap.Width, by + FBitmap.Height),
            FBitmap,
            Rect(0, 0, FBitmap.Width, FBitmap.Height),
            FBitmap.Canvas.Pixels[0, 0]);
      end;

      Pen.Style := psSolid;
      Pen.Color := clWindowFrame;
      if FDown then
      begin
         MoveTo(OuterHex[4].x, OuterHex[4].y);
         LineTo(OuterHex[5].x, OuterHex[5].y);
         LineTo(OuterHex[0].x, OuterHex[0].y);
      end
      else
      begin
         MoveTo(OuterHex[1].x, OuterHex[1].y);
         LineTo(OuterHex[2].x, OuterHex[2].y);
         LineTo(OuterHex[3].x, OuterHex[3].y);
      end;
      MoveTo(InnerHex[4].x, InnerHex[4].y);

      if FDown then
         Pen.Color := clBtnShadow
      else
         Pen.Color := clBtnHighlight;
      LineTo(InnerHex[5].x, InnerHex[5].y);
      LineTo(InnerHex[0].x, InnerHex[0].y);
      LineTo(InnerHex[1].x, InnerHex[1].y);

      if FDown then
         Pen.Color := clBtnHighlight
      else
         Pen.Color := clBtnShadow;
      LineTo(InnerHex[2].x, InnerHex[2].y);
      LineTo(InnerHex[3].x, InnerHex[3].y);
      LineTo(InnerHex[4].x, InnerHex[4].y);
   end;
end;

end.
