unit MabMFDlg;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, checklst, DB;

type
  TModifyFieldsDlg = class(TForm)
    HelpLabel: TLabel;
    FieldList: TCheckListBox;
    UpBtn: TButton;
    DownBtn: TButton;
    OkBtn: TButton;
    CancelBtn: TButton;
    RenameBtn: TButton;
    RenameEdit: TEdit;
    procedure FieldListClick(Sender: TObject);
    procedure UpBtnClick(Sender: TObject);
    procedure DownBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure RenameBtnClick(Sender: TObject);
    procedure FieldListKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure RenameEditEnter(Sender: TObject);
    procedure RenameEditExit(Sender: TObject);
    procedure RenameEditKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
  public
    { Public declarations }
  end;
  TAddFieldEvent = procedure(Sender: TObject; Field: TField; var CanAdd: Boolean) of object;
  TMabModifyFieldsDialog = class(TComponent)
  private
    FAllowRename: Boolean;
    FDataSource: TDataSource;
    FTitle: string;
    FOnAddField: TAddFieldEvent;
  public
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    function Execute: Boolean;

    constructor Create(AOwner: TComponent); override;
  published
    property AllowRename: Boolean read FAllowRename write FAllowRename default False;
    property DataSource: TDataSource read FDataSource write FDataSource;
    property Title: string read FTitle write FTitle;
    property OnAddField: TAddFieldEvent read FOnAddField write FOnAddField;
  end;

var
  ModifyFieldsDlg: TModifyFieldsDlg;

implementation

{$R *.DFM}

constructor TMabModifyFieldsDialog.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);

   FAllowRename := False;
   FDataSource := nil;
   FTitle := '';
   FOnAddField := nil;
end;

procedure TMabModifyFieldsDialog.Notification(AComponent: TComponent; Operation: TOperation);
begin
   if (Operation = opRemove) and (AComponent = DataSource) then
      DataSource := nil;
end;

function TMabModifyFieldsDialog.Execute: Boolean;
var
   i: Integer;
   CanAdd: Boolean;
begin
   if DataSource = nil then
      raise Exception.Create('DataSource must be specified')

   else
      with TModifyFieldsDlg.Create(Owner) do
      try
         RenameBtn.Visible := FAllowRename;

         if FTitle <> '' then Caption := FTitle;

         if DataSource.DataSet = nil then
            FieldList.Enabled := False
         else
         begin
            FieldList.Enabled := True;
            with DataSource.DataSet do
               for i := 0 to FieldCount - 1 do
               begin
                  CanAdd := True;
                  if Assigned(OnAddField) then OnAddField(Self, Fields[i], CanAdd);

                  if CanAdd then
                  begin
                     FieldList.Items.AddObject(Fields[i].DisplayName, Fields[i]);
                     FieldList.Checked[FieldList.Items.Count - 1] := Fields[i].Visible;
                  end;
               end;
         end;

         Result := (ShowModal = mrOk);

         if Result then
            with FieldList do
               for i := 0 to Items.Count - 1 do
               begin
                  TField(Items.Objects[i]).Visible := Checked[i];
                  TField(Items.Objects[i]).Index := i;

                  if FAllowRename then
                     TField(Items.Objects[i]).DisplayLabel := Items[i];
               end;
      finally
         Free;
      end;
end;

procedure TModifyFieldsDlg.FieldListClick(Sender: TObject);
begin
   with FieldList do
      if ItemIndex >= 0 then
      begin
         UpBtn.Enabled := (ItemIndex > 0);
         DownBtn.Enabled := (ItemIndex < Items.Count - 1);
         RenameBtn.Enabled := True;
      end
      else
      begin
         UpBtn.Enabled := False;
         DownBtn.Enabled := False;
         RenameBtn.Enabled := False;
      end;
end;

procedure TModifyFieldsDlg.UpBtnClick(Sender: TObject);
var
   NewIndex: Integer;
   WasChecked: Boolean;
begin
   with FieldList do
      if ItemIndex > 0 then
      begin
         NewIndex := ItemIndex - 1;
         WasChecked := Checked[ItemIndex];

         Items.Move(ItemIndex, NewIndex);
         Checked[NewIndex] := WasChecked;
         ItemIndex := NewIndex;

         FieldListClick(Sender);
      end;
end;

procedure TModifyFieldsDlg.DownBtnClick(Sender: TObject);
var
   NewIndex: Integer;
   WasChecked: Boolean;
begin
   with FieldList do
      if ItemIndex < Items.Count - 1 then
      begin
         NewIndex := ItemIndex + 1;
         WasChecked := Checked[ItemIndex];

         Items.Move(ItemIndex, NewIndex);
         Checked[NewIndex] := WasChecked;
         ItemIndex := NewIndex;

         FieldListClick(Sender);
      end;
end;

procedure TModifyFieldsDlg.FormCreate(Sender: TObject);
begin
   FieldListClick(Sender);
end;

procedure TModifyFieldsDlg.RenameBtnClick(Sender: TObject);
begin
   with FieldList do
      if ItemIndex >= 0 then
      begin
         RenameEdit.Top := Top + ItemRect(ItemIndex).Top;
         RenameEdit.Left := Left + ItemRect(ItemIndex).Left;
         RenameEdit.Width := Width;

         RenameEdit.Text := Items[ItemIndex];
         RenameEdit.Visible := True;
         ActiveControl := RenameEdit;
      end;
end;

procedure TModifyFieldsDlg.FieldListKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
   if RenameBtn.Visible and (Key = VK_F2) then
      RenameBtn.Click; 
end;

procedure TModifyFieldsDlg.RenameEditEnter(Sender: TObject);
begin
   OkBtn.Default := False;
   CancelBtn.Cancel := False;
end;

procedure TModifyFieldsDlg.RenameEditExit(Sender: TObject);
begin
   OkBtn.Default := True;
   CancelBtn.Cancel := True;
   RenameEdit.Visible := False;
end;

procedure TModifyFieldsDlg.RenameEditKeyPress(Sender: TObject;
  var Key: Char);
var
   WasChecked: Boolean;
begin
   if Ord(Key) = VK_RETURN then
   begin
      Key := #0;

      with FieldList do
      begin
         WasChecked := Checked[ItemIndex];
         Items[ItemIndex] := RenameEdit.Text;
         Checked[ItemIndex] := WasChecked;
      end;
      RenameEdit.Visible := False;
      ActiveControl := FieldList;
   end
   else if Ord(Key) = VK_ESCAPE then
   begin
      Key := #0;

      RenameEdit.Visible := False;
      ActiveControl := FieldList;
   end;
end;

end.
