unit MabRCSav;

interface

uses
   SysUtils, WinTypes, WinProcs, Classes, Graphics, Controls, Forms,
   MabRegSave, Registry;           

type
   TMabRegControlSave = class(TMabRegSave)
   private
      { Private declarations }
      { Private declarations }
      FControl: TControl;
      FRemember: TMabControlProperties;
   protected
      { Protected declarations }
      procedure Notification(AComponent: TComponent; Operation: TOperation); override;
   public
      { Public declarations }
      constructor Create(AOwner: TComponent); override;

      procedure Save;
      procedure Load;
   published
      { Published declarations }
      property Control: TControl read FControl write FControl;
      property Key;
      property Remember: TMabControlProperties read FRemember write FRemember;
   end;

implementation

constructor TMabRegControlSave.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);

   FRemember := [cpPosition, cpSize, cpVisible];
end;

procedure TMabRegControlSave.Notification(AComponent: TComponent; Operation: TOperation);
begin
   if (Operation = opRemove) and (AComponent = Control) then
      Control := nil;
end;

procedure TMabRegControlSave.Save;
var
   PName, PMesg: PChar;
   Section: string;
begin
   try
      if Key = '' then
         raise Exception.Create('Filename must be have a value');
      if Control = nil then
         raise Exception.Create('Control must be specified');

      with Control do
      begin
         Section := Name;

         if FRegistry.OpenKey(Key + Owner.ClassName, True) then
         begin
            if cpPosition in Remember then
            begin
               FRegistry.WriteInteger(Section, 'Top', Top);
               FRegistry.WriteInteger(Section, 'Left', Left);
            end;

            if cpSize in Remember then
            begin
               FRegistry.WriteInteger(Section, 'Height', Height);
               FRegistry.WriteInteger(Section, 'Width', Width);
            end;

            if cpVisible in Remember then
            begin
               FRegistry.WriteBool(Section, 'Visible', Visible);
            end;
         end;
         FRegistry.CloseKey;
      end;
   except
      on E: Exception do
      begin
         PName := StrAlloc(Length(Name) + 1);
         PMesg := StrAlloc(Length(E.Message) + 1);
         StrPCopy(PName, Name);
         StrPCopy(PMesg, E.Message);
         Application.MessageBox(PMesg, PName, MB_ICONEXCLAMATION);
         StrDispose(PName);
         StrDispose(PMesg);
      end;
   end;
end;

procedure TMabRegControlSave.Load;
var
   PName, PMesg: PChar;
   Section: string;
begin
   try
      if Key = '' then
         raise Exception.Create('Filename must be have a value');
      if Control = nil then
         raise Exception.Create('Control must be specified');

      with Control do
      begin
         Section := Name;

         if FRegistry.OpenKey(Key + Owner.ClassName, True) then
         begin
            if cpPosition in Remember then
            begin
               Top := FRegistry.ReadInteger(Section, 'Top', Top);
               Left := FRegistry.ReadInteger(Section, 'Left', Left);
            end;

            if cpSize in Remember then
            begin
               Height := FRegistry.ReadInteger(Section, 'Height', Height);
               Width := FRegistry.ReadInteger(Section, 'Width', Width);
            end;

            if cpVisible in Remember then
            begin
               Visible := FRegistry.ReadBool(Section, 'Visible', Visible);
            end;
         end;
         FRegistry.CloseKey;
      end;
   except
      on E: Exception do
      begin
         PName := StrAlloc(Length(Name) + 1);
         PMesg := StrAlloc(Length(E.Message) + 1);
         StrPCopy(PName, Name);
         StrPCopy(PMesg, E.Message);
         Application.MessageBox(PMesg, PName, MB_ICONEXCLAMATION);
         StrDispose(PName);
         StrDispose(PMesg);
      end;
   end;
end;

end.
