unit MabRDSav;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DB, MabRegSave;

type
  TMabRegDataSetSave = class(TMabRegSave)
  private
    { Private declarations }
      FDataSet: TDataSet;
      FRemember: TMabDSProperties;
  protected
    { Protected declarations }
      procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    { Public declarations }
      constructor Create(AOwner: TComponent); override;

      procedure Load;
      procedure Save;
  published
    { Published declarations }
      property DataSet: TDataSet read FDataSet write FDataSet;
      property Key;
      property Remember: TMabDSProperties read FRemember write FRemember;
  end;

implementation

constructor TMabRegDataSetSave.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);

   FRemember := [dpFieldDisplayWidths, dpFieldIndexes, dpFieldVisibles]
end;

procedure TMabRegDataSetSave.Notification(AComponent: TComponent; Operation: TOperation);
begin
   if (Operation = opRemove) and (AComponent = DataSet) then
      DataSet := nil;
end;

procedure TMabRegDataSetSave.Load;
var
   AList: TStringList;
   PName, PMesg: PChar;
   i: Integer;
begin
   try
      if Key = '' then
         raise Exception.Create('Key must be have a value');
      if DataSet = nil then
         raise Exception.Create('DataSet must be specified');

      with DataSet do
      begin
         {
         We must use AList here, rather than TTable's
         FieldCount and Fields[] properties, since the
         index of each field may change
         }
         AList := TStringList.Create;
         try
            GetFieldNames(AList);
            for i := 0 to AList.Count - 1 do
               with FindField(AList.Strings[i]) do
                  if FRegistry.OpenKey(Key + DataSet.Owner.ClassName + '\' + DataSet.Name + '\Fields', True) then
                  begin
                     if dpFieldDisplayWidths in Remember then
                        DisplayWidth := FRegistry.ReadInteger(FieldName, 'DisplayWidth', DisplayWidth);
                     if dpFieldVisibles in Remember then
                        Visible := FRegistry.ReadBool(FieldName, 'Visible', Visible);
                     if dpFieldIndexes in Remember then
                        Index := FRegistry.ReadInteger(FieldName, 'Index', Index);
                     if dpFieldLabels in Remember then
                        DisplayLabel := FRegistry.ReadString(FieldName, 'DisplayLabel', DisplayLabel);

                     FRegistry.CloseKey;
                  end;
         finally
            AList.Free;
         end;
      end;
   except
      on E: Exception do
      begin
         PName := StrAlloc(Length(Name) + 1);
         PMesg := StrAlloc(Length(E.Message) + 1);
         StrPCopy(PName, Name);
         StrPCopy(PMesg, E.Message);
         Application.MessageBox(PMesg, PName, MB_ICONEXCLAMATION);
         StrDispose(PName);
         StrDispose(PMesg);
      end;
   end;
end;

procedure TMabRegDataSetSave.Save;
var
   PName, PMesg: PChar;
   i: Integer;
   ASection: string;
begin
   try
      if Key = '' then
         raise Exception.Create('Key must be have a value');
      if DataSet = nil then
         raise Exception.Create('DataSet must be specified');

      with DataSet do
      begin
         ASection := Owner.ClassName + '.' + Name + '.Fields';
         for i := 0 to FieldCount - 1 do
            with Fields[i] do
               if FRegistry.OpenKey(Key + DataSet.Owner.ClassName + '\' + DataSet.Name + '\Fields', True) then
               begin
                  if dpFieldDisplayWidths in Remember then
                     FRegistry.WriteInteger(FieldName, 'DisplayWidth', DisplayWidth);
                  if dpFieldIndexes in Remember then
                     FRegistry.WriteInteger(FieldName, 'Index', Index);
                  if dpFieldVisibles in Remember then
                     FRegistry.WriteBool(FieldName, 'Visible', Visible);
                  if dpFieldLabels in Remember then
                     FRegistry.WriteString(FieldName, 'DisplayLabel', DisplayLabel);

                  FRegistry.CloseKey;
               end;
      end;
   except
      on E: Exception do
      begin
         PName := StrAlloc(Length(Name) + 1);
         PMesg := StrAlloc(Length(E.Message) + 1);
         StrPCopy(PName, Name);
         StrPCopy(PMesg, E.Message);
         Application.MessageBox(PMesg, PName, MB_ICONEXCLAMATION);
         StrDispose(PName);
         StrDispose(PMesg);
      end;
   end;
end;

end.
