unit MabRegSave;

interface

uses
   SysUtils, WinTypes, WinProcs, Classes, Registry, Graphics;

type
   TMabControlProperty = (cpPosition, cpSize, cpVisible);
   TMabDSProperty = (dpFieldDisplayWidths, dpFieldIndexes, dpFieldVisibles, dpFieldLabels);

   TMabControlProperties = set of TMabControlProperty;
   TMabDSProperties = set of TMabDSProperty;

   TMabRegSave = class(TComponent)
   private
      FKey: string;

      procedure SetKey(Value: string);
   protected
      FRegistry: TRegIniFile;
   public
      { Public declarations }
      constructor Create(AOwner: TComponent); override;
      destructor Destroy; override;

      procedure WriteFont(ARegistry: TRegIniFile; Section: string; Value: TFont);
      procedure ReadFont(ARegistry: TRegIniFile; Section: string; AFont: TFont);
   published
      property Key: string read FKey write SetKey;
   end;

implementation

constructor TMabRegSave.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);

   FRegistry := TRegIniFile.Create('');
   Key := 'Software\' + Owner.Name + '\';
end;

destructor TMabRegSave.Destroy;
begin
   FRegistry.Free;

   inherited Destroy;
end;

procedure TMabRegSave.SetKey(Value: string);
begin
   if Value = '' then
      raise Exception.Create('Key must be have a value')

   else if Value <> Key then
   begin
      if Copy(Value, Length(Value), 1) <> '\' then
         Value := Value + '\';

      FKey := Value;
   end;
end;

procedure TMabRegSave.WriteFont(ARegistry: TRegIniFile; Section: string; Value: TFont);
begin
   ARegistry.WriteString (Section, 'Name', Value.Name);
   ARegistry.WriteInteger(Section, 'Size',   Value.Size);
   ARegistry.WriteString (Section, 'Color',  ColorToString(Value.Color));
   ARegistry.WriteBool   (Section, 'Italic', (fsItalic in Value.Style));
   ARegistry.WriteBool   (Section, 'Bold',   (fsBold in Value.Style));
   ARegistry.WriteBool   (Section, 'Underline', (fsUnderline in Value.Style));
   ARegistry.WriteBool   (Section, 'Strikeout', (fsStrikeout in Value.Style));
   ARegistry.CloseKey;
end;

procedure TMabRegSave.ReadFont(ARegistry: TRegIniFile; Section: string; AFont: TFont);
var
   AStyle: TFontStyles;
begin
   AFont.Name  := ARegistry.ReadString(Section, 'Name', AFont.Name);
   AFont.Size  := ARegistry.ReadInteger(Section, 'Size', AFont.Size);
   AFont.Color := StringToColor(ARegistry.ReadString(Section, 'Color',
               ColorToString(AFont.Color)));

   AStyle := [];
   if ARegistry.ReadBool(Section, 'Italic', fsItalic in AFont.Style) then
      AStyle := AStyle + [fsItalic];

   if ARegistry.ReadBool(Section, 'Bold', fsBold in AFont.Style) then
      AStyle := AStyle + [fsBold];

   if ARegistry.ReadBool(Section, 'Underline', fsUnderline in AFont.Style) then
      AStyle := AStyle + [fsUnderline];

   if ARegistry.ReadBool(Section, 'Strikeout', fsStrikeout in AFont.Style) then
      AStyle := AStyle + [fsStrikeout];

   AFont.Style := AStyle;
end;

end.
