unit MabRSvCs;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  MabRegSave;

type
   TMabRegSaveControls = class;

   TMabSaveControl = class(TCollectionItem)
   private
      FControl: TControl;
      FRemember: TMabControlProperties;
   public
      constructor Create(Collection: TCollection); override;
      procedure Assign(Source: TPersistent); override;
      function GetDisplayName: string; override;
   published
      property Control: TControl read FControl write FControl;
      property Remember: TMabControlProperties read FRemember write FRemember;
   end;

   TMabSaveControls = class(TCollection)
   private
      FMabRegSaveControls: TMabRegSaveControls;

      function GetItem(Index: Integer): TMabSaveControl;
      procedure SetItem(Index: Integer; Value: TMabSaveControl);
   protected
      function GetOwner: TPersistent; override;
   public
      constructor Create(MabRegSaveControls: TMabRegSaveControls);
      function Add: TMabSaveControl;
      property Items[Index: Integer]: TMabSaveControl read GetItem write SetItem; default;
   end;

   TMabRegSaveControls = class(TMabRegSave)
   private
      { Private declarations }
      FOldFormCreate: TNotifyEvent;
      FOldFormDestroy: TNotifyEvent;

      FAutoSave: Boolean;
      FControls: TMabSaveControls;

      procedure FormCreate(Sender: TObject);
      procedure FormDestroy(Sender: TObject);
      procedure SetControls(Value: TMabSaveControls);
   protected
      { Protected declarations }
      procedure Notification(AComponent: TComponent; Operation: TOperation); override;
   public
      { Public declarations }
      constructor Create(AOwner: TComponent); override;
      destructor Destroy; override;

      procedure Save;
      procedure Load;
   published
      { Published declarations }
      property AutoSave: Boolean read FAutoSave write FAutoSave default True;
      property Controls: TMabSaveControls read FControls write SetControls;
   end;

implementation

constructor TMabSaveControl.Create(Collection: TCollection);
begin
   inherited Create(Collection);

   FRemember := [cpVisible];
end;

procedure TMabSaveControl.Assign(Source: TPersistent);
begin
   inherited Assign(Source);
   if Source is TMabSaveControl then
   begin
      FControl := TMabSaveControl(Source).Control;
      FRemember := TMabSaveControl(Source).FRemember;
   end;
end;

function TMabSaveControl.GetDisplayName: string;
begin
   if Control = nil then
      Result := inherited GetDisplayName
   else
      Result := Control.Name;
end;

constructor TMabSaveControls.Create(MabRegSaveControls: TMabRegSaveControls);
begin
  inherited Create(TMabSaveControl);
  FMabRegSaveControls:= MabRegSaveControls;
end;

function TMabSaveControls.Add: TMabSaveControl;
begin
  Result := TMabSaveControl(inherited Add);
end;

function TMabSaveControls.GetItem(Index: Integer): TMabSaveControl;
begin
  Result := TMabSaveControl(inherited GetItem(Index));
end;

function TMabSaveControls.GetOwner: TPersistent;
begin
  Result := FMabRegSaveControls;
end;

procedure TMabSaveControls.SetItem(Index: Integer; Value: TMabSaveControl);
begin
  inherited SetItem(Index, Value);
end;

constructor TMabRegSaveControls.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);

   FAutoSave := True;
   FControls := TMabSaveControls.Create(Self);

   if (Owner is TForm) and not (csDesigning in ComponentState) then
      with TForm(Owner) do
      begin
         FOldFormCreate := OnCreate;
         FOldFormDestroy := OnDestroy;

         OnCreate := Self.FormCreate;
         OnDestroy := Self.FormDestroy;
      end;
end;

destructor TMabRegSaveControls.Destroy;
begin
   if (Owner is TForm) and not (csDesigning in ComponentState) then
      with TForm(Owner) do
      begin
         OnCreate := FOldFormCreate;
         OnDestroy := FOldFormDestroy;
      end;
      
   FControls.Free;

   inherited Destroy;
end;

procedure TMabRegSaveControls.FormCreate(Sender: TObject);
begin
   if Assigned(FOldFormCreate) then
      FOldFormCreate(Sender);

   if AutoSave then
      Load;
end;

procedure TMabRegSaveControls.FormDestroy(Sender: TObject);
begin
   if Assigned(FOldFormDestroy) then
      FOldFormDestroy(Sender);

   if AutoSave then
      Save;
end;

procedure TMabRegSaveControls.SetControls(Value: TMabSaveControls);
begin
   FControls.Assign(Value);
end;

procedure TMabRegSaveControls.Notification(AComponent: TComponent; Operation: TOperation);
var
   i: Integer;
begin
   if Operation = opRemove then
   begin
      i := 0;
      while (i < Controls.Count) and (AComponent <> Controls[i].Control) do
         Inc(i);

      if i < Controls.Count then
         Controls[i].Free;;
   end;
end;

procedure TMabRegSaveControls.Save;
var
   i: Integer;
   Section: string;
begin
   try
      if Key = '' then
         raise Exception.Create('Key must be have a value');
      if Controls.Count = 0 then
         raise Exception.Create('No Controls specified');

      for i := 0 to Controls.Count - 1 do
         with Controls[i] do
            if Control <> nil then
            begin
               Section := Control.Name;

               if FRegistry.OpenKey(Key + Control.Owner.ClassName, True) then
               begin
                  if cpPosition in Remember then
                  begin
                     FRegistry.WriteInteger(Section, 'Top', Control.Top);
                     FRegistry.WriteInteger(Section, 'Left', Control.Left);
                  end;

                  if cpSize in Remember then
                  begin
                     FRegistry.WriteInteger(Section, 'Height', Control.Height);
                     FRegistry.WriteInteger(Section, 'Width', Control.Width);
                  end;

                  if cpVisible in Remember then
                  begin
                     FRegistry.WriteBool(Section, 'Visible', Control.Visible);
                  end;
               end;
               FRegistry.CloseKey;
            end;
   except
      on E: Exception do
         Application.MessageBox(PChar(E.Message), PChar(Name), MB_ICONEXCLAMATION);
   end;
end;

procedure TMabRegSaveControls.Load;
var
   i: Integer;
   Section: string;
begin
   try
      if Key = '' then
         raise Exception.Create('Key must be have a value');
      if Controls.Count = 0 then
         raise Exception.Create('No Controls specified');

      for i := 0 to Controls.Count - 1 do
         with Controls[i] do
            if Control <> nil then
            begin
               Section := Control.Name;

               if FRegistry.OpenKey(Key + Control.Owner.ClassName, True) then
               begin
                  if cpPosition in Remember then
                  begin
                     Control.Top := FRegistry.ReadInteger(Section, 'Top', Control.Top);
                     Control.Left := FRegistry.ReadInteger(Section, 'Left', Control.Left);
                  end;

                  if cpSize in Remember then
                  begin
                     Control.Height := FRegistry.ReadInteger(Section, 'Height', Control.Height);
                     Control.Width := FRegistry.ReadInteger(Section, 'Width', Control.Width);
                  end;

                  if cpVisible in Remember then
                  begin
                     Control.Visible := FRegistry.ReadBool(Section, 'Visible', Control.Visible);
                  end;
               end;
               FRegistry.CloseKey;
            end;
   except
      on E: Exception do
         Application.MessageBox(PChar(E.Message), PChar(Name), MB_ICONEXCLAMATION);
   end;
end;

end.
