unit MabRSvDs;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DB, MabRegSave;

type
   TMabRegSaveDataSets = class;

   TMabSaveDataSet = class(TCollectionItem)
   private
      FDataSet : TDataSet;
      FRemember: TMabDSProperties;
   public
      constructor Create(Collection: TCollection); override;
      procedure Assign(Source: TPersistent); override;
      function GetDisplayName: string; override;
   published
      property DataSet: TDataSet read FDataSet write FDataSet;
      property Remember: TMabDSProperties read FRemember write FRemember;
   end;

   TMabSaveDataSets = class(TCollection)
   private
      FMabRegSaveDataSets: TMabRegSaveDataSets;

      function GetItem(Index: Integer): TMabSaveDataSet;
      procedure SetItem(Index: Integer; Value: TMabSaveDataSet);
   protected
      function GetOwner: TPersistent; override;
   public
      constructor Create(MabRegSaveDataSets: TMabRegSaveDataSets);
      function Add: TMabSaveDataSet;
      property Items[Index: Integer]: TMabSaveDataSet read GetItem write SetItem; default;
   end;

   TMabRegSaveDataSets = class(TMabRegSave)
   private
      { Private declarations }
      FDataSets: TMabSaveDataSets;

      procedure SetDataSets(Value: TMabSaveDataSets);
   protected
      { Protected declarations }
      procedure Notification(AComponent: TComponent; Operation: TOperation); override;
   public
      { Public declarations }
      constructor Create(AOwner: TComponent); override;
      destructor Destroy; override;

      procedure Save;
      procedure Load;
   published
      { Published declarations }
      property DataSets: TMabSaveDataSets read FDataSets write SetDataSets;
   end;

implementation

constructor TMabSaveDataSet.Create(Collection: TCollection);
begin
   inherited Create(Collection);

   FRemember := [dpFieldDisplayWidths, dpFieldIndexes, dpFieldVisibles]
end;

procedure TMabSaveDataSet.Assign(Source: TPersistent);
begin
   inherited Assign(Source);
   if Source is TMabSaveDataSet then
   begin
      FDataSet := TMabSaveDataSet(Source).FDataSet;
      FRemember := TMabSaveDataSet(Source).FRemember;
   end;
end;

function TMabSaveDataSet.GetDisplayName: string;
begin
   if DataSet = nil then
      Result := inherited GetDisplayName
   else
      Result := DataSet.Name;
end;

constructor TMabSaveDataSets.Create(MabRegSaveDataSets: TMabRegSaveDataSets);
begin
  inherited Create(TMabSaveDataSet);
  FMabRegSaveDataSets := MabRegSaveDataSets;
end;

function TMabSaveDataSets.Add: TMabSaveDataSet;
begin
  Result := TMabSaveDataSet(inherited Add);
end;

function TMabSaveDataSets.GetItem(Index: Integer): TMabSaveDataSet;
begin
  Result := TMabSaveDataSet(inherited GetItem(Index));
end;

function TMabSaveDataSets.GetOwner: TPersistent;
begin
  Result := FMabRegSaveDataSets;
end;

procedure TMabSaveDataSets.SetItem(Index: Integer; Value: TMabSaveDataSet);
begin
  inherited SetItem(Index, Value);
end;

constructor TMabRegSaveDataSets.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);

   FDataSets := TMabSaveDataSets.Create(Self);
end;

destructor TMabRegSaveDataSets.Destroy;
begin
   FDataSets.Free;

   inherited Destroy;
end;

procedure TMabRegSaveDataSets.SetDataSets(Value: TMabSaveDataSets);
begin
   FDataSets.Assign(Value);
end;

procedure TMabRegSaveDataSets.Notification(AComponent: TComponent; Operation: TOperation);
var
   i: Integer;
begin
   if Operation = opRemove then
   begin
      i := 0;
      while (i < DataSets.Count) and (AComponent <> DataSets[i].DataSet) do
         Inc(i);

      if i < DataSets.Count then
         DataSets[i].Free;;
   end;
end;

procedure TMabRegSaveDataSets.Save;
var
   i, j: Integer;
   ASection: string;
begin
   try
      if Key = '' then
         raise Exception.Create('Key must be have a value');
      if DataSets.Count = 0 then
         raise Exception.Create('No DataSets specified');

      for i := 0 to DataSets.Count - 1 do
         with DataSets[i] do
            if DataSet <> nil then
            begin
               ASection := Owner.ClassName + '.' + Name + '.Fields';
               for j := 0 to DataSet.FieldCount - 1 do
                  with DataSet.Fields[j] do
                     if FRegistry.OpenKey(Key + DataSet.Owner.ClassName + '\' + DataSet.Name + '\Fields', True) then
                     begin
                        if dpFieldDisplayWidths in Remember then
                           FRegistry.WriteInteger(FieldName, 'DisplayWidth', DisplayWidth);
                        if dpFieldIndexes in Remember then
                           FRegistry.WriteInteger(FieldName, 'Index', Index);
                        if dpFieldVisibles in Remember then
                           FRegistry.WriteBool(FieldName, 'Visible', Visible);
                        if dpFieldLabels in Remember then
                           FRegistry.WriteString(FieldName, 'DisplayLabel', DisplayLabel);

                        FRegistry.CloseKey;
                     end;
            end;
   except
      on E: Exception do
         Application.MessageBox(PChar(E.Message), PChar(Name), MB_ICONEXCLAMATION);
   end;
end;

procedure TMabRegSaveDataSets.Load;
var
   i, j: Integer;
   AList: TStringList;
begin
   try
      if Key = '' then
         raise Exception.Create('Key must be have a value');
      if DataSets.Count = 0 then
         raise Exception.Create('No DataSets specified');

      for i := 0 to DataSets.Count - 1 do
         with DataSets[i] do
            if DataSet <> nil then
            begin
               {
               We must use AList here, rather than TTable's
               FieldCount and Fields[] properties, since the
               index of each field may change
               }
               AList := TStringList.Create;
               try
                  DataSet.GetFieldNames(AList);
                  for j := 0 to AList.Count - 1 do
                     with DataSet.FindField(AList.Strings[j]) do
                        if FRegistry.OpenKey(Key + DataSet.Owner.ClassName + '\' + DataSet.Name + '\Fields', True) then
                        begin
                           if dpFieldDisplayWidths in Remember then
                              DisplayWidth := FRegistry.ReadInteger(FieldName, 'DisplayWidth', DisplayWidth);
                           if dpFieldVisibles in Remember then
                              Visible := FRegistry.ReadBool(FieldName, 'Visible', Visible);
                           if dpFieldIndexes in Remember then
                              Index := FRegistry.ReadInteger(FieldName, 'Index', Index);
                           if dpFieldLabels in Remember then
                              DisplayLabel := FRegistry.ReadString(FieldName, 'DisplayLabel', DisplayLabel);

                           FRegistry.CloseKey;
                        end;
               finally
                  AList.Free;
               end;
            end;
   except
      on E: Exception do
         Application.MessageBox(PChar(E.Message), PChar(Name), MB_ICONEXCLAMATION);
   end;
end;

end.
